/*
 *  cmdline.c: Copyright 2004 Phil Whineray
 *
 *  This file is part of x70talk.
 *
 *  x70talk is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  x70talk is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with x70talk; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "cmdline.h"
#include "phone.h"

const char *argp_program_version = X70TALK_VERSION;
const char *argp_program_bug_address = "<phil@slightly-cracked.com>";
error_t argp_err_exit_status = 1;

static char doc[] =
  "x70talk -- A program to help communicate with your Panasonic X70";

static char args_doc[] = "list\n\
write phonebook_first_name\n\
read\n\
clear-all-phonebook-entries\n\
import file-name\n\
info\n\
set-time\n\
delete ID\n\
get ID\n\
put FILE";

static struct argp_option options[] = {
  {"output",    'o', "FILE", 0, "Use FILE instead of the default file name" },
  {"verbose",   'v', 0,      0, "Count blocks whilst uploading/downloading" },
  {"folder",    'f', "NUM",  0, "Use phone folder NUM 1-8 (4 = User 1) not "
                                                           DEFAULT_FOLDER },
  {"entry",     'e', "NUM",  0, "Use entry NUM" },
  {"last",      'l', "NAME", 0, "Last name = NAME" },
  {"home",      'h', "NUM",  0, "Home number = NUM" },
  {"work",      'w', "NUM",  0, "Work number = NUM" },
  {"mobile",    'm', "NUM",  0, "Mobile number = NUM" },
  {"email",     '@', "ADDR", 0, "Email = ADDR" },
  {"device",    'd', "DEV" , 0, "Use DEV instead of " DEFAULT_DEVICE },
  {"bluetooth", 'b', "ADDR", 0, "Use ADDR instead of " DEFAULT_DEVICE },
  {"trace",     't', 0,      0, "Display reads and writes on stderr" },
  { 0 }
};

static error_t parse_opt (int key, char *arg, struct argp_state *state)
{
  /* Get the INPUT argument from `argp_parse', which we
     know is a pointer to our arguments structure. */
  struct arguments *arguments = state->input;

  switch (key) {
    case 't':
      arguments->debug = 1;
      break;
    case 'v':
      arguments->verbose = 1;
      break;
    case 'o':
      arguments->output_file = arg;
      break;
    case 'b':
      fprintf(stderr,"warning: --bluetooth deprecated - use --device\n");
      fprintf(stderr,"         this option is redundant and will be removed\n");
      arguments->device_name = arg;
      break;
    case 'd':
      arguments->device_name = arg;
      break;
    case 'f':
      arguments->phone_folder = atoi(arg);
      if (arguments->phone_folder < 1 || arguments->phone_folder > 8) {
          fprintf(stderr, "error: phone folder must be beteen 1 and 8\n");
          argp_usage (state);
      }
      break;
    case 'e':
      arguments->entry_number = atoi(arg);
      if (arguments->entry_number < 1 || arguments->entry_number > 200) {
          fprintf(stderr, "error: phonebook entry must be beteen 1 and 200\n");
          argp_usage (state);
      }
      break;
    case 'n':
      arguments->name_first = arg;
      break;
    case 'l':
      arguments->name_last = arg;
      break;
    case 'h':
      arguments->number_home = arg;
      break;
    case 'w':
      arguments->number_work = arg;
      break;
    case 'm':
      arguments->number_mobile = arg;
      break;
    case '@':
      arguments->email_home = arg;
      break;
    case ARGP_KEY_ARG:
      if (state->arg_num == 0) {
        if (strcasecmp(arg, "LIST") == 0 || strcasecmp(arg, "L") == 0)
           arguments->command = COMMAND_LIST;
        else if (strcasecmp(arg, "READ") == 0 || strcasecmp(arg, "R") == 0)
           arguments->command = COMMAND_READ;
        else if (strcasecmp(arg, "WRITE") == 0 || strcasecmp(arg, "W") == 0)
           arguments->command = COMMAND_WRITE;
        else if (strcasecmp(arg, "CLEAR-ALL-PHONEBOOK-ENTRIES") == 0)
           arguments->command = COMMAND_CLEAR;
        else if (strcasecmp(arg, "IMPORT") == 0)
           arguments->command = COMMAND_IMPORT;
        else if (strcasecmp(arg, "DELETE") == 0 || strcasecmp(arg, "D") == 0)
           arguments->command = COMMAND_DELETE;
        else if (strcasecmp(arg, "GET") == 0 || strcasecmp(arg, "G") == 0)
           arguments->command = COMMAND_GET;
        else if (strcasecmp(arg, "PUT") == 0 || strcasecmp(arg, "P") == 0)
           arguments->command = COMMAND_PUT;
        else if (strcasecmp(arg, "INFO") == 0 || strcasecmp(arg, "I") == 0)
           arguments->command = COMMAND_INFO;
        else if (strcasecmp(arg, "SET-TIME") == 0)
           arguments->command = COMMAND_SETTIME;
        else {
          fprintf(stderr, "error: unrecognised command\n");
          argp_usage (state);
        }
      } else if (state->arg_num == 1
               && (arguments->command == COMMAND_GET
                || arguments->command == COMMAND_PUT
                || arguments->command == COMMAND_DELETE)) {
        arguments->file_or_id = arg;
      } else if (state->arg_num == 1
               && (arguments->command == COMMAND_WRITE
                || arguments->command == COMMAND_IMPORT)) {
        arguments->name_first = arg;
      } else {
        fprintf(stderr, "error: Too many arguments\n");
        argp_usage (state);
      }
      break;

    case ARGP_KEY_END:
      if (state->arg_num == 0) {
        argp_usage (state);
      } else if (state->arg_num == 1
       && (arguments->command != COMMAND_LIST &&
           arguments->command != COMMAND_READ &&
           arguments->command != COMMAND_CLEAR &&
           arguments->command != COMMAND_SETTIME &&
           arguments->command != COMMAND_INFO)) {
        fprintf(stderr, "error: Too few arguments\n");
        argp_usage (state);
      } else if (state->arg_num == 2
       && (arguments->command != COMMAND_GET &&
           arguments->command != COMMAND_DELETE &&
           arguments->command != COMMAND_WRITE &&
           arguments->command != COMMAND_IMPORT &&
           arguments->command != COMMAND_PUT)) {
        fprintf(stderr, "error: Too few arguments\n");
        argp_usage (state);
      }
      break;
                                                                                
    default:
      return ARGP_ERR_UNKNOWN;
  }
  return 0;
}

static struct argp argp = { options, parse_opt, args_doc, doc };

void parse_commandline(int argc, char *argv[], struct arguments *args)
{
  if (argp_parse(&argp, argc, argv, 0, NULL, args))
    exit(1);
}
