/*
 *  main.c: Copyright 2003-2004 Phil Whineray
 *
 *  This file is part of x70talk.
 *
 *  x70talk is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  x70talk is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with x70talk; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Panasonic X70 Phone Manager utility
 */
#include "phone.h"
#include "cmdline.h"

#define do_error(msg, val,buf) {\
    if (val < 0) { \
      perror(msg); \
    } else { \
      if (strcmp("+CME ERROR: 14", buf) == 0) \
        fprintf(stdout, "%s: %s (NOT ON MAIN MENU)\n", msg, phone_error(val)); \
      else \
        fprintf(stdout, "%s: %s (%s)\n", msg, phone_error(val), buf); \
    } return 1; }

#define numel(x) (sizeof(x)/sizeof((x)[0]))

/*
 * Work round strtok taking out repeats. Real fix is to not use strtok.
 */
static char *mystrtok(char *in, char *val)
{
  static char *buf;
  char *first;

  if (in)
    buf = in;

  if (*buf == 0)
    return NULL;

   first = buf;
   while (*buf && *buf != *val)
     buf++;
   if (*buf)
     *buf++ = 0;
   return first;
}

int main(int argc, char *argv[])
{
  int i, fd, count;
  char errbuf[4096]; /* Needs to be big; whole misunderstood line goes here */
  struct phone_file_info files[1024]; /* How many files... */
  int numfiles, got_count, max_count, total_slots;
  int default_phone_folder = 0;
  char name[1024], *ext;
  struct arguments args;

  memset(&args, 0, sizeof(args));
  if (getenv("X70_DEVICE"))
    args.device_name = getenv("X70_DEVICE");
  else
    args.device_name = DEFAULT_DEVICE;

/*
 * We can't set the initial value of args.phone_folder here because
 * the behaviour differs between functions - for uploading we will
 * always want the "default" folder if no other is specified, but
 * for listing files no -f option means list from all folders, not
 * just the default one.
 */
  if (getenv("X70_FOLDER"))
    default_phone_folder = atoi(getenv("X70_FOLDER"));
  else
    default_phone_folder = atoi(DEFAULT_FOLDER);

  parse_commandline(argc, argv, &args);

  if (args.verbose)
    x70_verbose = 1;
  if (args.debug)
    serial_debug = 1;

  *errbuf = 0;

  fd = serial_open(args.device_name);
  if (fd < 0) {
    fprintf(stderr, "Error opening %s: %s\n", args.device_name,strerror(errno));
    exit(1);
  }

  i = x70_init(fd, errbuf);
  if (i == SERIAL_TIMEOUT) {
    x70_kick(fd, errbuf);
    i = x70_init(fd, errbuf);
  }
  if (i)
    do_error("X70 init", i, errbuf);

  if (getenv("X70_SETTIME") && *getenv("X70_SETTIME") == 'Y') {
    i = x70_set_time(fd, errbuf);
    if (i)
      do_error("X70 set time", i, errbuf);
  }

  switch (args.command) {
    case COMMAND_WRITE:
      i = x70_write_phonebook_entry(fd, errbuf, "ME", args.entry_number,
                                    args.name_first,
                                    args.name_last?args.name_last:"",
                                    args.number_home?args.number_home:"",
                                    args.number_work?args.number_work:"",
                                    args.number_mobile?args.number_mobile:"",
                                    args.email_home?args.email_home:"");
      if (i)
        do_error("X70 write phonebook", i, errbuf);
    break;

    case COMMAND_IMPORT:
      {
        char inbuf[1024];
        FILE *fp;
        char buf[64];
        printf("Import phonebook is destructive - are you sure? (type yes to continue): ");
        fgets(buf, sizeof(buf), stdin);
        buf[sizeof(buf)-1] = 0;
        if (buf[3] == '\n') buf[3] = 0;
        if (strcmp(buf, "yes") != 0) {
          printf("Aborted.\n");
          exit(1);
        }
        fp = fopen(args.name_first, "r");
        if (!fp) {
          perror(args.name_first);
          return 1;
        }
        got_count = 0;
        while (fgets(inbuf, sizeof(inbuf), fp)) {
          char *entry_number, *name_first, *name_last, *number_home;
          char *number_work, *number_mobile, *email_home;
          inbuf[strlen(inbuf)-1] = 0;
  
          got_count++;
          entry_number  = mystrtok(inbuf, ",");
          name_first    = mystrtok(NULL, ",");
          name_last     = mystrtok(NULL, ",");
          number_home   = mystrtok(NULL, ",");
          number_work   = mystrtok(NULL, ",");
          number_mobile = mystrtok(NULL, ",");
          email_home    = mystrtok(NULL, ",");
      
          printf("Importing %d\n", got_count);
          i = x70_write_phonebook_entry(fd, errbuf, "ME", atoi(entry_number),
                                        name_first?name_first:"",
                                        name_last?name_last:"",
                                        number_home?number_home:"",
                                        number_work?number_work:"",
                                        number_mobile?number_mobile:"",
                                        /*email_home?email_home:*/"");
          if (i)
            do_error("X70 write phonebook", i, errbuf);
        }
      }
    break;

    case COMMAND_CLEAR:
      {
        char buf[64];
        printf("Clear phonebook - are you sure? (type yes to continue): ");
        fgets(buf, sizeof(buf), stdin);
        buf[sizeof(buf)-1] = 0;
        if (buf[3] == '\n') buf[3] = 0;
        if (strcmp(buf, "yes") != 0) {
          printf("Aborted.\n");
          exit(1);
        }
      }
      i = x70_count_phonebook_entries(fd,errbuf,"ME", &max_count, &total_slots);
      if (i)
        do_error("X70 count phonebook", i, errbuf);
      for (got_count = 0, count = 1;
           got_count < max_count && count <= total_slots; count++) {
        if (!args.entry_number || args.entry_number == count) {
           char name_first[256];
           char name_last[256];
           char number_home[256];
           char number_work[256];
           char number_mobile[256];
           char email_home[256];
           i = x70_read_phonebook_entry(fd, errbuf, "ME", count,
                                        name_first, name_last, number_home,
                                        number_work, number_mobile, email_home);
           if (i == 0) {
             got_count++;
             printf("Clearing %d of %d\n", got_count, max_count);
             i = x70_write_phonebook_entry(fd, errbuf, "ME", count,
                                    "", "", "", "", "", "");
             if (i)
               do_error("X70 remove phonebook entry", i, errbuf);
           }
           else if (i != PHONE_NO_SUCH_ENTRY || args.entry_number)
             do_error("X70 read phonebook", i, errbuf);
        }
      }
    break;

    case COMMAND_READ:
      i = x70_count_phonebook_entries(fd,errbuf,"ME", &max_count, &total_slots);
      if (i)
        do_error("X70 count phonebook", i, errbuf);
      for (got_count = 0, count = 1;
           got_count < max_count && count <= total_slots; count++) {
        if (!args.entry_number || args.entry_number == count) {
           char name_first[256];
           char name_last[256];
           char number_home[256];
           char number_work[256];
           char number_mobile[256];
           char email_home[256];
           i = x70_read_phonebook_entry(fd, errbuf, "ME", count,
                                        name_first, name_last, number_home,
                                        number_work, number_mobile, email_home);
           if (i == 0) {
             printf("%d,%s,%s,%s,%s,%s,%s\n", count, name_first, name_last,
                         number_home, number_work, number_mobile, email_home);
             got_count++;
           }
           else if (i != PHONE_NO_SUCH_ENTRY || args.entry_number)
             do_error("X70 read phonebook", i, errbuf);
        }
      }
    break;

    case COMMAND_LIST:
      i = x70_files(fd, errbuf, args.phone_folder, files, numel(files),
                                                          &numfiles);
      if (i)
        do_error("X70 files", i, errbuf);

      for (i = 0; i < numfiles; i++) {
        printf("%6d (%6ld bytes) %d/%s.%s\n", files[i].id, files[i].size,
                            files[i].folder, files[i].name, files[i].extension);
      }
    break;

    case COMMAND_INFO:
      {
      int battery, signal;
      i = x70_info(fd, errbuf, &battery, &signal);
      if (i)
        do_error("X70 info", i, errbuf);
      printf("battery: %d%%\nsignal: %d%%\n", battery, signal);
      }
    break;

    case COMMAND_SETTIME:
      i = x70_set_time(fd, errbuf);
      if (i)
        do_error("X70 set time", i, errbuf);
      printf("Phone time set from system clock\n");
    break;

    case COMMAND_DELETE:
      i = x70_delete(fd, errbuf, atoi(args.file_or_id));
      if (i)
        do_error("X70 delete", i, errbuf);
    break;

    case COMMAND_GET:
      i = x70_download(fd, errbuf, atoi(args.file_or_id), args.output_file);
      if (i)
        do_error("X70 download", i, errbuf);
    break;

    case COMMAND_PUT:
      if (args.output_file)
        strcpy(name, args.output_file);
      else {
        char tmpname[1024], *c;
        strncpy(tmpname, args.file_or_id, sizeof(tmpname));
        tmpname[sizeof(tmpname)-1] = 0;
        for (c = tmpname+strlen(tmpname) - 1; c > tmpname; c--) {
          if (*c == '/' || *c == '\\') {
            c++;
            break;
          }
        }
        strcpy(name, c);
      }
      ext = name;
      while (*ext && *ext != '.')
        ext++;
      if (*ext)
        *ext++ = 0;

      i = x70_upload(fd, errbuf,
                     args.phone_folder?args.phone_folder:default_phone_folder,
                     name, ext, args.file_or_id);
      if (i)
        do_error("X70 upload", i, errbuf);
    break;
  }

  serial_close(fd);
  return 0;
}
