#ifndef _PHONE_H
#define _PHONE_H

/*
 *  phone.h: Copyright 2003-2004 Phil Whineray
 *
 *  This file is part of x70talk.
 *
 *  x70talk is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  x70talk is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with x70talk; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define X70TALK_VERSION "x70talk 0.1.4-cvs"

/*
 * If the user does not specify a device, we will try to use this one
 */
#define DEFAULT_DEVICE "/dev/bluetooth/rfcomm/0"

/*
 * If the user does not specify a phone folder, we will try to use this one
 */
#define DEFAULT_FOLDER "8" /* User 5 folder (valid = 1 to 8) */

/*
 * Maximum time the code will wait for a response from the phone before
 * giving up
 */
#define TIMEOUT 3

/*
 * Error codes used by all phone functions
 *
 */
#define SERIAL_SHORT_WRITE             1
#define SERIAL_TIMEOUT                 2
#define SERIAL_BAD_PARAMETER           3
#define PHONE_INIT_FAILED            101
#define PHONE_TYPE_FAILED            102
#define PHONE_UNEXPECTED_RESPONSE    103
#define PHONE_FILE_OPEN              104
#define PHONE_NO_SUCH_ENTRY          105
#define PHONE_CHECKSUM               106
#define PHONE_SIZE_MISMATCH          107
#define PHONE_NO_SPACE_LEFT          108
#define AT_MORE                      201
#define AT_RESPONSE_TOO_BIG          202
#define AT_ERROR                     203
#define AT_MEMORY_ERROR              204
const char *phone_error(int error_code);

/*
 * Information about the files stored on the phone
 */
struct phone_file_info {
   int  folder;
   char name[256];
   char extension[16];
   int  id;
   long size;
   char checksum[3]; /* Used during downloads */
};

/*
 * Information about the phonebook entries stored on the phone
 */
struct phone_book_info {
   int  entry;
   char name_first[256];
   char name_last[256];
   char number_home[256];
   char number_work[256];
   char number_mobile[256];
   char email_home[256];
};

/*
 * General serial commands
 */
extern int serial_debug;
int  serial_open(const char *path);
void serial_close(int fd);
int  serial_write(int fd, const char *buf);
int  serial_write_raw(int fd, const char *buf, int len);
int  serial_read(int fd, char *buf, int buflen);
int  serial_read_raw(int fd, char *buf, int buflen);
int  serial_flush(int fd, int character, int count);
int  serial_clear(int fd);

/*
 * AT command set
 *
 * The "len" argument is needed in order to be able to pass unicode values
 * through command. If it is 0, the length wull be calculated as per normal.
 *
 * Return values: 0  = OK
 *                1  = MORE (submit using at_more())
 *                <0 = ERROR
 *
 * responses are allocated on the heap, and should be freed using at_free()
 */
struct at_response_list {
  struct at_response_list *next;
  int eol_char;
  char line[1024];
};

int at_run(int fd, const char *command, int len,
           struct at_response_list **response);
int at_more(int fd, const char *command, int len,
            struct at_response_list **response);
void at_free(struct at_response_list **response);


/*
 * Individual phone commands
 */
extern int x70_verbose;
void x70_kick(int fd, char *errbuf);
int  x70_init(int fd, char *errbuf);
int  x70_set_time(int fd, char *errbuf);
int  x70_info(int fd, char *errbuf, int *battery, int *signal);
int  x70_files(int fd, char *errbuf, int folder, struct phone_file_info *files,
              int max_files, int *used_files);
int  x70_download(int fd, char *errbuf, int id, const char *local_filename);
int  x70_upload(int fd, char *errbuf, int dirid, const char *remote_filename,
                const char *remote_ext, const char *local_filename);
int  x70_delete(int fd, char *errbuf, int fileid);
int x70_count_phonebook_entries(int fd, char *errbuf, const char *mem,
                                int *count, int *max);
int x70_write_phonebook_entry(int fd, char *errbuf, const char *mem,
                              int entry, const char *first, const char *last,
                                         const char *home, const char *work,
                                         const char *mobile, const char *email);
int x70_read_phonebook_entry(int fd, char *errbuf, const char *mem,
                             int entry, char *first, char *last, char *home,
                             char *work, char *mobile, char *email);

/*
 * Adler32 commands
 */
unsigned long adler32(unsigned long adler, const unsigned char *buf, int len);

/*
 * Standard includes used by one or more files
 */
#include "config.h"
#include <stdio.h>
#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif
#if HAVE_ARGP_H
#include <argp.h>
#endif
#if HAVE_CTYPE_H
#include <ctype.h>
#endif
#if HAVE_ERRNO_H
#include <errno.h>
#endif
#if HAVE_LIBGEN_H
#include <libgen.h>
#endif
#if HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#if HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#if HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif
#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_BLUETOOTH_BLUETOOTH_H
#include <bluetooth/bluetooth.h>
#endif
#if HAVE_BLUETOOTH_RFCOMM_H
#include <bluetooth/rfcomm.h>
#endif
#if HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#if HAVE_TIME_H
#include <time.h>
#endif
#if HAVE_WINDOWS_H
#include <windows.h>
#endif
#if HAVE_WINSOCK2_H
#include <winsock2.h>
#endif
#if HAVE_WS2BTH_H
#include <Ws2bth.h>
#endif

#endif
