/*
 *  x70.c: Copyright 2003-2004 Phil Whineray
 *
 *  This file is part of x70talk.
 *
 *  x70talk is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  x70talk is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with x70talk; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * implement a set of phone features for the Panasonic X70
 */
#include "phone.h"

#define start_buffer_or_fail(expected,got,errcode,errbuf) { \
                            if (strncmp(expected, got, strlen(expected))!=0) { \
                              strcpy(errbuf, got); \
                              return errcode; \
                            } }

#define xval(c) (((c) >= 'A' && (c) <= 'F')?(c)-'A'+10:(c)-'0')
#define xhigh(c) (((c) >> 4) & 0xf)
#define xlow(c) ((c) & 0xf)

#define highchar(c) ((xhigh(c) > 9)?(xhigh(c)+'A'-10):(xhigh(c)+'0'))
#define lowchar(c) ((xlow(c) > 9)?(xlow(c)+'A'-10):(xlow(c)+'0'))

int x70_verbose = 0;

void x70_kick(int fd, char *errbuf)
{
/*
 * try to poke the phone into life - this appears to involve sending
 * ^Z followed by several 0x0b to the phone, it will eventually start to
 * respond, after which we can start using an enhanced AT command set.
 */
  serial_flush(fd, 032, 1);
  serial_flush(fd, 0x0b, 4);
  serial_clear(fd);
}

int x70_init(int fd, char *errbuf)
{
  struct at_response_list *resp;
  int i;

/* Read anything left on the line */
  serial_write(fd, "");
  serial_clear(fd);

/* Initialise the connection */
  i = at_run(fd, "ATZ", 0, &resp);
  if (i)
    return i;
  at_free(&resp);

/* Check the phone type */
  i = at_run(fd, "AT+CGMM", 0, &resp);
  if (i)
    return i;

  if (!resp || strcmp(resp->line, "X70") != 0)
    return PHONE_TYPE_FAILED;
  at_free(&resp);

  return 0;
}

static int parse_book_info(struct phone_book_info *book, const char *buf)
{
  const char *c = buf+9;
  char *o, *end;
  char numbuf[128];

  o = numbuf;
  end = numbuf+sizeof(numbuf)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  book->entry = atol(numbuf);
  if (*c)
    c++;

  o = book->number_home;
  end = book->number_home+sizeof(book->number_home)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = numbuf; /* Number type - discarded */
  end = numbuf+sizeof(numbuf)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = book->name_first;
  end = book->name_first+sizeof(book->name_first)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = book->name_last;
  end = book->name_last+sizeof(book->name_last)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = book->number_work;
  end = book->number_work+sizeof(book->number_work)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = numbuf; /* Number type - discarded */
  end = numbuf+sizeof(numbuf)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = book->number_mobile;
  end = book->number_mobile+sizeof(book->number_mobile)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = numbuf; /* Number type - discarded */
  end = numbuf+sizeof(numbuf)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  o = book->email_home;
  end = book->email_home+sizeof(book->email_home)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  if (*c)
    c++;

  return 0;
}

static int parse_file_info(struct phone_file_info *files, const char *buf)
{
  const char *c = buf+8;
  char *o, *end;
  char numbuf[128];

  o = files->name;
  end = files->name+sizeof(files->name)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;

  if (*c)
    c++;

  o = files->extension;
  end = files->extension+sizeof(files->extension)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;

  if (*c)
    c++;

  o = numbuf;
  end = numbuf+sizeof(numbuf)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  files->size = atol(numbuf);
  if (*c)
    c++;

  o = numbuf;
  end = numbuf+sizeof(numbuf)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;
  files->id = atoi(numbuf);

  if (*c)
    c++;

  o = files->checksum;
  end = files->checksum+sizeof(files->checksum)-1;
  while (*c && *c != ',') {
    if (*c == '"') {
      c++;
      continue;
    }
    *o = *c;
    if (o < end)
      o++;
    c++;
  }
  *o = 0;

  if (*c)
    c++;

  return 0;
}

static int x70_files_dir(int fd, char *errbuf, struct phone_file_info *files,
                         int max_files, int *used_files,
                         const char *command, int folder)
{
  struct at_response_list *resp;
  int i;

  *used_files = 0;

  i= at_run(fd, command, 0, &resp);
  if (i)
    return i;
  if (!resp)
      return 0; /* Empty folder */

  while (resp) {
    start_buffer_or_fail("+CUPL: ",resp->line,PHONE_UNEXPECTED_RESPONSE,errbuf);

    if (*used_files < max_files) {
      i = parse_file_info(files+*used_files, resp->line);
      files[*used_files].folder = folder;
      if (i)
          return i;
      (*used_files)++;
    }
    resp = resp->next;
  }
  return 0;
}

int x70_files(int fd, char *errbuf, int folder, struct phone_file_info *files,
              int max_files, int *used_files)
{
  int i, num, dirnum;
  char cmd[44];

  *used_files = 0;
  if (folder) {
    sprintf(cmd, "AT+CUPL=0,%d", folder);
    i = x70_files_dir(fd, errbuf, files, max_files, &num, cmd, folder);
    if (i)
      return i;
    *used_files = num;
    return 0;
  }

  for (dirnum = 1; dirnum <= 8; dirnum++) {
    sprintf(cmd, "AT+CUPL=0,%d", dirnum);
    i = x70_files_dir(fd, errbuf, files, max_files, &num, cmd, dirnum);
    if (i)
      return i;
    *used_files += num;
    files += num;
    max_files -= num;
  }

  return 0;
}

static void calculate_checksum(char *sumval, const char *addr, int count)
{
  unsigned long sum = adler32(0, addr, count);
  sprintf(sumval, "%02lX", sum & 0xff);
}

static void tobin(char *bin, char *hex)
{
  int val;
  while (isxdigit(hex[0]) && isxdigit(hex[1])) {
    val = xval(hex[0]) * 16 + xval(hex[1]);
    *bin++ = val;
    hex+=2;
  }
}

static void tohex(char *hex, char *bin, int blockbytes)
{
  while (blockbytes) {
    *hex++ = highchar(*bin);
    *hex++ = lowchar(*bin);
    blockbytes--;
    bin++;
  }
  *hex = 0;
}

/*
 * The phone looks to take unicode names in various places... Should
 * we ever wish to deal with esoteric encodings this will need
 * replacing. For basic 7-bit ASCII char set it's fine however.
 */
static void airquotes_unicode(char *out, const char *in)
{
  while (*in) {
    *out++ = 0;
    *out++ = *in++;
  }
  *out++ = 0;
  *out++ = 0;
}

int x70_info(int fd, char *errbuf, int *battery, int *signal)
{
  struct at_response_list *resp;
  int got_signal, got_battery;
  int i;

  i = at_run(fd, "AT+CSQ", 0, &resp);
  if (i)
    return i;

  if (!resp)
    signal = 0;
  else {
    start_buffer_or_fail("+CSQ", resp->line, PHONE_UNEXPECTED_RESPONSE, errbuf);
    sscanf(resp->line, "+CSQ: %d,0", &got_signal);
    if (got_signal > 31 || got_signal < 1)
      got_signal = 0;
    else
      got_signal = ((float)got_signal)/0.31;
  }
  at_free(&resp);

  i = at_run(fd, "AT+CBC", 0, &resp);
  if (i)
    return i;
  if (!resp)
    return PHONE_UNEXPECTED_RESPONSE;
  sscanf(resp->line, "+CBC: 0,%d", &got_battery);
  at_free(&resp);

  if (battery)
    *battery = got_battery;
  if (signal)
    *signal = got_signal;
  
  return 0;
}

int x70_set_time(int fd, char *errbuf)
{
  struct tm *tm;
  time_t now;
  struct at_response_list *resp;
  char cmdbuf[1024];
  int i;

  time(&now);
  tm = localtime(&now);
  sprintf(cmdbuf, "AT+CCLK=%02d/%02d/%02d,%02d:%02d:%02d",
          tm->tm_year-100, tm->tm_mon,  tm->tm_mday, 
          tm->tm_hour, tm->tm_min,  tm->tm_sec);

  i = at_run(fd, cmdbuf, 0, &resp);
  if (i)
    return i;

  return 0;
}

int x70_download(int fd, char *errbuf, int id, const char *local_filename)
{
  struct at_response_list *resp;
  FILE *fp = NULL;
  int i;
  char cmd[80], sum[80];
  char outbuf[4096];
  struct phone_file_info info;

  if (local_filename) {
    fp = fopen(local_filename, "wb");
    if (!fp) {
      strcpy(errbuf, local_filename);
      return PHONE_FILE_OPEN;
    }
  }

  sprintf(cmd, "AT+CUPR=%d", id);
  i = at_run(fd, cmd, 0, &resp);
  if (i)
    return i;
  if (!resp) {
    sprintf(errbuf, "file id %d", id);
    return PHONE_NO_SUCH_ENTRY;
  }

  while (resp) {
    start_buffer_or_fail("+CUPR: ",resp->line,PHONE_UNEXPECTED_RESPONSE,errbuf);
    i = parse_file_info(&info, resp->line);
    if (i)
      return i;
    if (!fp) {
      char fbuf[sizeof(info.name)+sizeof(info.extension)];
      sprintf(fbuf, "%s.%s", info.name, info.extension);
      fp = fopen(fbuf, "wb");
      if (!fp) {
        strcpy(errbuf, fbuf);
        return PHONE_FILE_OPEN;
      }
    }
    if (x70_verbose)
      fprintf(stderr, "Reading block %ld\n", info.size+1);
    resp = resp->next;
    if (!resp)
      return PHONE_UNEXPECTED_RESPONSE;
    if (strlen(resp->line) != info.id * 2) {
      sprintf(errbuf, "expected %d got %d", info.id*2, strlen(resp->line));
      return PHONE_SIZE_MISMATCH;
    }
    tobin(outbuf, resp->line);
    calculate_checksum(sum, outbuf, info.id);
    if (*sum && strcmp(sum, info.checksum) != 0) {
      sprintf(errbuf, "expected %s got %s", info.checksum, sum);
      return PHONE_CHECKSUM;
    }
    fwrite(outbuf, 1, info.id, fp);

    resp = resp->next;
  }
  
  fclose(fp);
  return 0;
}

int x70_upload(int fd, char *errbuf, int dirid, const char *remote_filename,
               const char *remote_ext, const char *local_filename)
{
  struct at_response_list *resp;
  FILE *fp;
  char cmd[4096], sum[80];
  char inbuf[4096];
  char buf[4096];
  int bytes, blockbytes, counter, maxblocks;
  int i, len;

  fp = fopen(local_filename, "rb");
  if (!fp) {
    strcpy(errbuf, local_filename);
    return PHONE_FILE_OPEN;
  }
  fseek(fp, 0, SEEK_END);
  bytes = ftell(fp);
  fseek(fp, 0, SEEK_SET);

  strcpy(cmd, "AT+CUPS=,\"");
  len = strlen(cmd);
  airquotes_unicode(cmd+len, remote_filename);
  len += strlen(remote_filename) * 2;
  sprintf(cmd+len, "\",\"%s\",%d,%d", remote_ext, bytes, dirid);
  len += strlen(cmd+len);
  i = at_run(fd, cmd, len, &resp);
  if (i)
    return i;
  at_free(&resp);

  counter = 0;
  maxblocks = bytes/256 + ((bytes%256)?1:0);
  while (bytes > 0) {
    blockbytes = (bytes < 256)?bytes:256;
    fread(inbuf, 1, blockbytes, fp);

    calculate_checksum(sum, inbuf, blockbytes);
    sprintf(cmd, "AT+CUPW=%d,%d,%s", (bytes<257)?65535:counter,
                                     blockbytes, sum);
    if (x70_verbose)
      fprintf(stderr, "Writing block %d of %d\n", counter+1, maxblocks);
    i = at_run(fd, cmd, 0, &resp);
    if (i != AT_MORE)
      return PHONE_UNEXPECTED_RESPONSE;
    at_free(&resp);

    tohex(buf, inbuf, blockbytes);

    i = at_more(fd, buf, 0, &resp);
    if (i != 0)
      return i;
    at_free(&resp);

    bytes -= blockbytes;
    counter++;
  }
  
  fclose(fp);
  serial_write(fd, "");
  serial_clear(fd);
  return 0;
}

int x70_delete(int fd, char *errbuf, int fileid)
{
  struct at_response_list *resp;
  char cmd[1024];
  int i;

  sprintf(cmd, "AT+CUPS=%d,\"\",\"\",,", fileid);
  i = at_run(fd, cmd, 0, &resp);
  if (i)
    return i;
  at_free(&resp);
  return 0;
}

static char *number_type(const char *number)
{
  if (*number == '+') /* International */
    return "145";
  if (strncmp(number, "00", 2) == 0) /* European international */
    return "145";
  if (strncmp(number, "010", 3) == 0) /* USA international */
    return "145";
  if (*number)
    return "129";
  return "";
}

int x70_count_phonebook_entries(int fd, char *errbuf, const char *mem,
                                int *count, int *max)
{
  struct at_response_list *resp;
  int got_count, got_max;
  int i;

  i = at_run(fd, "AT+CMEPBR?", 0, &resp);
  if (i)
    return i;
  if (!resp)
    return PHONE_UNEXPECTED_RESPONSE;
  sscanf(resp->line, "+CMEPBR: \"ME\",%d,%d", &got_count, &got_max);

  if (count)
    *count = got_count;
  if (max)
    *max = got_max;
  return 0;
}

int x70_write_phonebook_entry(int fd, char *errbuf, const char *mem,
                              int entry,
                              const char *first, const char *last,
                              const char *home, const char *work,
                              const char *mobile, const char *email)
{
  struct at_response_list *resp;
  char cmd[4096];
  int err, len;
  char *work_email = ""; /* not supported by this command */

/*
 * Note: the commands below are not general: to access the SIM memory we need
 * to check the "mem" value, and if it is "SM" then we really need to do some
 * clever stuff like:
 *   AT+CPBS="SM"
 *   AT+CPBR=?
 *   AT+CPMS=?
 *   AT+CPMS="SM"
 *   AT+CPBS=?
 *
 * before getting to the meat of the problem. It may well be that the
 * phone uses a different format for these records too.
 */
  if (!entry) {
    int i,count,max;
    err =  x70_count_phonebook_entries(fd, errbuf, mem, &count, &max);
    if (err != 0)
      return err;
    if (count == max)
      return PHONE_NO_SPACE_LEFT;

    /* Look for a free entry */
    for (i = count+1; !entry && i <= max; i++) {
      err = x70_read_phonebook_entry(fd, errbuf, mem, i, NULL, NULL,
                                     NULL, NULL, NULL, NULL);
      if (err == PHONE_NO_SUCH_ENTRY) {
        entry = i;
      } else if (err != 0) {
        return err;
      }
    }

    /* Look for a free entry */
    for (i = 1; !entry && i <= count; i++) {
      err = x70_read_phonebook_entry(fd, errbuf, mem, i, NULL, NULL,
                                     NULL, NULL, NULL, NULL);
      if (err == PHONE_NO_SUCH_ENTRY) {
        entry = i;
      } else if (err != 0) {
        return err;
      }
    }

    /* Should have been trapped by count==max, but just to be safe */
    if (!entry)
      return PHONE_NO_SPACE_LEFT;
  }


/* Set up command and wait for response */
  sprintf(cmd, "AT+C%sPBW=%d,", mem, entry);
  len = strlen(cmd);
  sprintf(cmd+len, "\"%s\",%s,\"", home, number_type(home));
  len = len + strlen(cmd+len);
  airquotes_unicode(cmd+len, first);
  len = len + strlen(first) * 2;
  strcpy(cmd+len, "\",");
  len = len + strlen(cmd+len);

  err = at_run(fd, cmd, len, &resp);
  if (err != AT_MORE)
    return PHONE_UNEXPECTED_RESPONSE;

  /* Write rest of information */
  strcpy(cmd, "\"");
  len = strlen(cmd);
  airquotes_unicode(cmd+len, last);
  len = len + strlen(last) * 2;
  strcpy(cmd+len, "\",");
  len = len + strlen(cmd+len);
  sprintf(cmd+len, "\"%s\",%s,", work, number_type(work));
  len = len + strlen(cmd+len);
  sprintf(cmd+len, "\"%s\",%s,", mobile, number_type(mobile));
  len = len + strlen(cmd+len);
  sprintf(cmd+len, "\"%s\",\"%s\",,\"\"", email, work_email);
  len = len + strlen(cmd+len);
  err = at_more(fd, cmd, len, &resp);
  if (err != AT_MORE)
    return PHONE_UNEXPECTED_RESPONSE;

  err = at_more(fd, "", 0, &resp);
  if (err)
    return err;

/*
 * Who knows why these are necesary? There seems to be rubbish left on
 * the line by the phone which will screw us if:
 *   a) we don't send another character
 *   b) clear the stuff coming back (some of what we wrote, but no newline)
 */
  serial_write(fd, "");
  serial_clear(fd);

  return 0;
}

int x70_read_phonebook_entry(int fd, char *errbuf, const char *mem,
                             int entry,
                             char *first, char *last,
                             char *home, char *work,
                             char *mobile, char *email)
{
  struct at_response_list *resp;
  struct phone_book_info pb;
  char cmd[1024];
  int i, err;

  memset(&pb, 0, sizeof(pb));

  if (first)
    *first = 0;
  if (last)
    *last = 0;
  if (home)
    *home = 0;
  if (work)
    *work = 0;
  if (mobile)
    *mobile = 0;
  if (email)
    *email = 0;

/*
 * Note: the sprintf below is wrong: to access the SIM memory we need to
 * check the "mem" value, and if it is "SM" then we really need to do some
 * clever stuff as for writing.
 */

  sprintf(cmd, "AT+C%sPBR=%d", mem, entry);
  i = at_run(fd, cmd, 0, &resp);
  if (i)
    return i;

  if (!resp) {
    sprintf(errbuf, "entry %d", entry);
    return PHONE_NO_SUCH_ENTRY;
  }

  start_buffer_or_fail("+CMEPBR: ",resp->line,PHONE_UNEXPECTED_RESPONSE,errbuf);

  /* Parse information */
  err = parse_book_info(&pb, resp->line);
  if (first)
    strcpy(first, pb.name_first);
  if (last)
    strcpy(last, pb.name_last);
  if (home)
    strcpy(home, pb.number_home);
  if (work)
    strcpy(work, pb.number_work);
  if (mobile)
    strcpy(mobile, pb.number_mobile);
  if (email)
    strcpy(email, pb.email_home);

  return 0;
}
